
def show_nearby_atoms (structure, atom_of_interest) :
  def format_xyz (xyz) :
    return "(%.3f, %.3f, %.3f)" % tuple(xyz)
  from cctbx import crystal
  from cctbx import xray
  assert(isinstance(structure, xray.structure))
  asu_mappings = structure.asu_mappings(buffer_thickness = 3)
  pair_asu_table = crystal.pair_asu_table(asu_mappings = asu_mappings)
  pair_asu_table.add_all_pairs(distance_cutoff = 3)
  raw_mappings = asu_mappings.mappings()
  raw_pair_table = pair_asu_table.table() # access to the raw structures
  unit_cell = structure.unit_cell()
  scatterers = structure.scatterers()
  for i_seq, scatterer in enumerate(scatterers):
    rt_mx_i_inv = asu_mappings.get_rt_mx(i_seq, 0).inverse()
    label = scatterer.element_symbol().strip()
    if label == atom_of_interest: # we are only interested in one atom type
      site_cart = unit_cell.orthogonalize(scatterer.site)
      print "%s : %s" % (scatterer.label, format_xyz(site_cart))
      for j_seq, j_syms in raw_pair_table[i_seq].items():
        j_site_frac = scatterers[j_seq].site
        j_site_cart = unit_cell.orthogonalize(j_site_frac)
        print "  neighbor (%d): %s at %s" % (j_seq, scatterers[j_seq].label,
          format_xyz(j_site_cart))
        for sym_group in j_syms:
          for j_sym in sym_group: # j_sym is an index into
            rt_mx = rt_mx_i_inv.multiply(asu_mappings.get_rt_mx(j_seq, j_sym))
            print "    j_sym:", j_sym, "i_sym_op:", raw_mappings[j_seq][j_sym].i_sym_op()
            print "      To be transformed by %s (symmetry #%s)" % \
              (rt_mx, j_sym)
            j_site_symm = rt_mx * j_site_frac
            j_site_symm_cart = unit_cell.orthogonalize(j_site_symm)
            print "      into %s" % format_xyz(j_site_symm_cart)
            distance = unit_cell.distance(scatterer.site, j_site_symm)
            print "      distance: %.3f Angstrom" % distance

def run () :
  import iotbx.pdb.fetch
  hierarchy, xrs = iotbx.pdb.fetch.load_pdb_structure("2wll")
  show_nearby_atoms(xrs, "K")

if (__name__ == "__main__") :
  run()
